'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = trackDeviceConnection;

var _sensor = require('../API/sensors/sensor.model');

var _sensor2 = _interopRequireDefault(_sensor);

var _input = require('../data-streams/input');

var _input2 = _interopRequireDefault(_input);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _env = require('../config/env');

var _env2 = _interopRequireDefault(_env);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('SH_BE:device-connected');

var DEVICE_INFO_EVENT = 'device-info';

function trackDeviceConnection() {

    _input2.default.stream.filter(function (message) {
        return message.event === DEVICE_INFO_EVENT;
    }).subscribe(saveDevice, onError);

    function saveDevice(data) {
        _sensor2.default.find({
            mqttId: data.device
        }, function (error, records) {
            if (records.length) {
                debug('Device already added');
            } else {
                var sensorModel = Object.assign({
                    mqttId: data.device,
                    executor: isExecutor(data.value.type),
                    value: data.value,
                    valueUpdated: Date.now()
                }, data.value);

                var device = new _sensor2.default(sensorModel);

                device.saveAsync().then(function (device) {
                    return onDeviceAdded(device);
                }).catch(onError);
            }
        });
    }

    function onDeviceAdded(device) {
        debug('Added device: \'' + device + '\' to db');

        if (_env2.default.plugAndPlay) {
            _input2.default.write({
                event: 'device-add',
                data: device
            });
        }
    }

    function onError(error) {
        debug('Error: \'' + error + '\' occured');
    }

    function isExecutor(type) {
        var exectorTypes = ['relay', 'servo'];
        return exectorTypes.indexOf(type) !== -1;
    }
}
module.exports = exports['default'];
//# sourceMappingURL=deviceConnected.js.map
