'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _env = require('../config/env');

var _env2 = _interopRequireDefault(_env);

var _mqtt = require('mqtt');

var _mqtt2 = _interopRequireDefault(_mqtt);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _input = require('../data-streams/input');

var _input2 = _interopRequireDefault(_input);

var _output = require('../data-streams/output');

var _output2 = _interopRequireDefault(_output);

var _convertMqttMessageToEvent = require('./event/convert-mqtt-message-to-event');

var _convertMqttMessageToEvent2 = _interopRequireDefault(_convertMqttMessageToEvent);

var _eventType = require('./event/event-type');

var _topicPrefix = require('./topic-prefix');

var _uniqueQueue = require('./unique-queue');

var _uniqueQueue2 = _interopRequireDefault(_uniqueQueue);

var _lock = require('./lock');

var _lock2 = _interopRequireDefault(_lock);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @file Reads config for the mqqt broker address,
 * connects to it and provides event data to the data hub
 */

var debug = (0, _debug2.default)('SH_BE:mqtt-client');

var MQTT_DEVICE_INFO_TIMEOUT = 60 * 1000;

// Create a client connection

var client = _mqtt2.default.connect({
    host: _env2.default.mqtt.hostname,
    port: _env2.default.mqtt.port,
    auth: _env2.default.mqtt.username + ':' + _env2.default.mqtt.password
});

var deviceInfoPublishQueue = (0, _uniqueQueue2.default)();
var deviceInfoPublishQueueLock = (0, _lock2.default)();
var deviceInfoPublishQueueClearTimeout = 0;

client.on('connect', onMqttConnect).on('message', onMqttMessage);

function onMqttConnect() {
    client.subscribe(_topicPrefix.INPUT + '#');
    _output2.default.stream.subscribe(onInputEvent);
}

function onMqttMessage(topic, rawMessage) {
    debug('MQTT >>. Got message: topic \'' + topic + '\', message: \'' + rawMessage.toString() + '\'');
    var event = (0, _convertMqttMessageToEvent2.default)(topic, rawMessage);
    if (event.event === _eventType.DEVICE_INFO) {
        deviceInfoPublishQueue.remove(event.device);
        deviceInfoPublishQueueLock.unlock();
        publishDeviceInfoEvent();
    }
    _input2.default.write(event);
    debug('Added to input stream event: \'' + JSON.stringify(event) + '\'');
}

function onInputEvent(event) {
    if (event.event === _eventType.DEVICE_STATUS) {
        client.publish(_topicPrefix.OUTPUT + event.device, event.value);
    } else if (event.event === _eventType.DEVICE_INFO) {
        publishDeviceInfoEvent(event.device);
    } else {
        debug('Unknown output stream event received: \'' + JSON.stringify(event) + '\'');
    }
}

function publishDeviceInfoEvent(device) {

    device && deviceInfoPublishQueue.enqueue(device);

    if (!deviceInfoPublishQueueLock.isLocked) {
        clearTimeout(deviceInfoPublishQueueClearTimeout);
        if (!deviceInfoPublishQueue.isEmpty) {
            deviceInfoPublishQueueLock.lock();
            client.publish(_topicPrefix.OUTPUT + _eventType.DEVICE_INFO, deviceInfoPublishQueue.dequeue());
            deviceInfoPublishQueueClearTimeout = setTimeout(clearDeviceInfoPublishQueue, MQTT_DEVICE_INFO_TIMEOUT);
        }
    }
}

function clearDeviceInfoPublishQueue() {
    deviceInfoPublishQueue.clear();
    deviceInfoPublishQueueLock.unlock();
    debug('Queue cleared by timeout');
}

exports.default = client;
module.exports = exports['default'];
//# sourceMappingURL=client.js.map
